import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..models.key_scope import KeyScope

T = TypeVar("T", bound="UserKeyInfo")


@attr.s(auto_attribs=True)
class UserKeyInfo:
    """
    Attributes:
        key_id (str):
        created_at (datetime.datetime):
        scope (KeyScope): An enumeration.
        alias (str):
    """

    key_id: str
    created_at: datetime.datetime
    scope: KeyScope
    alias: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        key_id = self.key_id
        created_at = self.created_at.isoformat()

        scope = self.scope.value

        alias = self.alias

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "key_id": key_id,
                "created_at": created_at,
                "scope": scope,
                "alias": alias,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        key_id = d.pop("key_id")

        created_at = isoparse(d.pop("created_at"))

        scope = KeyScope(d.pop("scope"))

        alias = d.pop("alias")

        user_key_info = cls(
            key_id=key_id,
            created_at=created_at,
            scope=scope,
            alias=alias,
        )

        user_key_info.additional_properties = d
        return user_key_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
